'Gertesoftware ... : Modulare Kleinsteuerung MKS
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Programm ......... : Beispiel(e) fr den Betrieb der Ein- und Ausgangskanle
' Prog.-Sprache ... : BASCOM
' Prog.-Inhalt .... : - Konfiguration von Lichteffekten
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Programmierer .... : Ingolf Bauer (ingolf.bauer@nexgo.de)
' Prog.-Version ... : 1.0
' Prog.-Datum ..... : 01.03.2005
'-------------------------------------------------------------------------------
'
'Historie
'========
'V 1.0 (nderungen zur Version 0.1):
'- - - - - - - - - - - - - - - - - -
'00. 01.03.2005 - Programmierung
'01. Erforderlicher Programmspeicher des MCs (Programm-Flashspeicher): <15%
'
'V 0.1
'01. 01.12.2004 - Konzepterarbeitung
'-------------------------------------------------------------------------------
'
'Hinweise
'========
'01.  Allgemeines
'01.1 Im Men von BASCOM (Compiler-Chip) vor dem Compilieren umfangreicher Pro-
'     gramme mindestens einstellen mit:
'     > HW Stack 32   Soft Stack 32   Framesize 32
'     Es kommt sonst zu undefinierten Sprngen oder falschen Anzeigen.
'
'02.  Einstellung der FUSE-Bits:
'02.1 Frequenzeinstellung (extern, Quarz)
'     > CKSEL1 bis CKSEL3 mit Haken versehen
'     Im Lieferzustand hat nur CKSEL0 die Markierung.
'02.2 JTAG(EN)
'     > JTAGEN mit Haken versehen
'     Beim ATmega16 wird das JTAG(EN) Flag defaultmig auf Enable gesetzt. Dies
'     bedeutet, da das JTAG-Interface (prozenahe Datenabfrage) aktiviert ist.
'     Dann kann aber Port C fr keine anderen Aufgaben genutzt werden!
'     Wie und ob das Flag programmig verndert werden kann, ist mir nicht be-
'     kannt. Auf alle Flle kann es ausgeschaltet werden, wenn unter "Configura-
'     tion" beim AVR-Programmer ein Haken bei JTAGEN gesetzt wird.
'
'03.  Anschluschema
'03.1 ATMega16
'     Der Port D ist als digitaler Eingangsport mit 8 Kanlen, die Ports A und
'     C als digitale Ausgangsports mit insgesamt 16 Kanlen initialisiert.
'
'                         --------------|_|--------------
'             LCD-HGB --> | 1 [PB0]             [PA0] 40| >--- DA1.1
'             LCD-HGB --< | 2 [PB1]             [PA1] 39| >--- DA1.2
'             LCD-RS ---< | 3 [PB2]             [PA2] 38| >--- DA1.3
'             LCD-E  ---< | 4 [PB3]             [PA3] 37| >--- DA1.4
'             LCD-DB4 --< | 5 [PB4]             [PA4] 36| >--- DA1.5
'    PRG -- / LCD-DB5 >-< | 6 [PB5, MOSI]       [PA5] 35| >--- DA1.6
'    PRG -- / LCD-DB6 <-< | 7 [PB6, MISO]       [PA6] 34| >--- DA1.7
'    PRG -- / LCD-DB7 >-< | 8 [PB7, SCK]        [PA7] 33| >--- DA1.8
'    PRG -- / Taster ---> | 9 [/Reset]         [AREF] 32| ==== P5V (analog)
'             P5V ======= |10 [VCC]            [AGND] 31| ==== GND (analog)
'             GND ======= |11 [GND]            [AVCC] 30| ==== P5V (analog)
'             Quarz +++++ |12 [XTAL2]           [PC7] 29| >--- DA2.8
'             Quarz +++++ |13 [XTAL1]           [PC6] 28| >--- DA2.7
'            DE1.1 -----> |14 [PD0]             [PC5] 27| >--- DA2.6
'            DE1.2 -----> |15 [PD1]             [PC4] 26| >--- DA2.5
'            DE1.3 -----> |16 [PD2]             [PC3] 25| >--- DA2.4
'            DE1.4 -----> |17 [PD3]             [PC2] 24| >--- DA2.3
'            DE1.5 -----> |18 [PD4]             [PC1] 23| >--- DA2.2
'            DE1.6 -----> |19 [PD5]             [PC0] 22| >--- DA2.1
'            DE1.7 -----> |20 [PD6]             [PD7] 21| <--- DE1.8
'                         -------------------------------
'
'     PRG     ............ : Programmierinterface (parallele Schnittstelle)
'     LCD-xxx ............ : LCD-Anzeige-Modul
'     Quarz .............. : Quarzanschlu (16,0 MHz)
'     P5V/ (analog) ...... : P 5 V der Versorgungsspannung (stabilisiert)
'     GND/ (analog) ...... : Masse der Versorgungsspannung
'
'04.  I/O-Ports
'04.1 Abkrzungen
'     DAx.y .............. : digitaler Ausgang Port.Kanal
'     DEx.y .............. : digitaler Eingang Port.Kanal
'     LCD ................ : LCD-Anzeige
'     PRG ................ : Programmierung ber LPT1 mit Adapterkabel
'     HGB ................ : Hintergrundbeleuchtung der LCD-Anzeige
'
'04.2 Belegung der AVR-Anschlsse
'     >Port
'     Port A  Pin  Typ | Gert  Signal ||
'     --------------------------------------------------------
'      PA0     40  DA  | -      extern ||
'      PA1     39  DA  | -      extern ||
'      PA2     38  DA  | -      extern ||
'      PA3     37  DA  | -      extern ||
'      PA4     36  D4  | -      extern ||
'      PA5     35  DA  | -      extern ||
'      PA6     34  DA  | -      extern ||
'      PA7     33  DA  | -      extern ||
'
'     Port B  Pin  Typ | Gert  Signal ||  Typ | Gert  Signal
'     --------------------------------------------------------
'      PB0     1   DA  | LCD    HGB    ||
'      PB1     2   DE  | LCD    HGB    ||
'      PB2     3   DA  | LCD    RS     ||
'      PB3     4   DA  | LCD    E      ||
'      PB4     5   DA  | LCD    DB4    ||
'      PB5     6   DA  | LCD    DB5    ||  DE  | PRG    MOSI
'      PB6     7   DA  | LCD    DB6    ||  DA  | PRG    MISO
'      PB7     8   DA  | LCD    DB7    ||  DE  | PRG    SCK
'
'     Port C  Pin  Typ | Gert  Signal ||
'     --------------------------------------------------------
'      PC0     22  DA  | -      extern ||
'      PC1     23  DA  | -      extern ||
'      PC2     24  DA  | -      extern ||
'      PC3     25  DA  | -      extern ||
'      PC4     26  D4  | -      extern ||
'      PC5     27  DA  | -      extern ||
'      PC6     28  DA  | -      extern ||
'      PC7     29  DA  | -      extern ||
'
'     Port D  Pin  Typ | Gert  Signal ||
'     --------------------------------------------------------
'      PD0     14  DE  | -      extern ||
'      PD1     15  DE  | -      extern ||
'      PD2     16  DE  | -      extern ||
'      PD3     17  DE  | -      extern ||
'      PD4     18  DE  | -      extern ||
'      PD5     19  DE  | -      extern ||
'      PD6     20  DE  | -      extern ||
'      PD7     21  DE  | -      extern ||
'
'     Sonstige
'             Pin  Typ | Gert  Signal ||  Typ | Gert  Signal
'     --------------------------------------------------------
'              9   DE  | Taster /RESET ||   DE | PRG    /RESET
'-------------------------------------------------------------------------------
'
'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'                          P R O G R A M M S T A R T
'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'
'Definition
   $regfile = "m16def.dat"                       'Prozessor ... : ATMega16
   $crystal = 3680000                            'Quarz ....... : 16,0 MHz
'-------------------------------------------------------------------------------
'
'
'         E I N S T E L L U N G   F  R   D E N   T E S T B E T R I E B
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Achtung:
'========
' Soll der Compiler fr den Simulationsbetrieb Wartezeiten bergehen, kann im
' Testbetrieb der folgende Befehl ($sim) aktiviert werden.
' $sim
'
'
'LCD Konfiguration
'-bei angeschlossener LCD-Anzeige erforderlich
'=LCD-Typ ..... : 2 Zeilen, 16 Zeichen
'-Datenport ... : Port B wird neben den Anschlssen zur Programmierung auch
'                 zur Datenbertragung ans LCD-Modul benutzt
   Config Lcd = 16 * 2
   Config Lcdpin = Pin , Db4 = Portb.4 , Db5 = Portb.5 , Db6 = Portb.6 , _
          Db7 = Portb.7 , E = Portb.3 , Rs = Portb.2
   Cls                                           'LCD-Anzeige lschen
   Cursor Off                                    'Kursor aus

'Deklaration
'=Variable
'-Flag/ Zhler/ Merker
   Dim M_ba As Byte                              'Merker: Betriebsart
   Dim Z_out_int As Integer                      'Zhler: Ausgang      (16 Bit)
   Dim Z_out1_byt As Byte                        'Zhler: Ausgang 1    ( 8 Bit)
   Dim Z_out2_byt As Byte                        'Zhler: Ausgang 2    ( 8 Bit)
   Dim T_warte As Integer                        'Timer: Wartezeit
   Dim Z_umschaltung As Integer                  'Zhler fr Umschaltung

'Initialisierung
'=I/O-Ports
'==Port A
'-digitale Ausgnge (Port 1)
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
   Ddra = &B1111_1111                            'PA0-7 Aus
'-Pullup-Widerstand (aktiv [1], passiv [0])
'--wird bentigt, wenn Initialisierung als Eingangsport
'   Porta = &B0000_0000                           'PA0-7 passiv

'==Port B
'-reserviert fr LCD Anzeige und Programmierschnittstelle
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
   Ddrb = &B1111_1111                            'PB0-7 Aus
'-Pullup-Widerstand (aktiv [1], passiv [0])
'--wird bentigt, wenn Initialisierung als Eingangsport
'   Portb = &B0000_0000                           'PB0-7 passiv

'==Port C
'-digitale Ausgnge (Port 2)
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
   Ddrc = &B1111_1111                            'PC0-7 Aus
'-Pullup-Widerstand (aktiv [1], passiv [0])
'--wird bentigt, wenn Initialisierung als Eingangsport
'   Portc = &B0000_0000                           'PC0-7 passiv

'==Port D
'-digitale Eingnge (Port 3)
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
   Ddrd = &B0000_0000                            'PD0-7 Ein
'-Pullup-Widerstand (aktiv [1], passiv [0])
   Portd = &B1111_1111                           'PD0-7 aktiv

'=Werte
   Z_out1_byt = 128                              'Startwert
   Z_out2_byt = 1                                'Startwert
   M_ba = 0                                      'Startwert


'Programm
Do
'Umschaltzeit kann hier fr alle Betriebsarten umgeschaltet werden
   T_warte = 100                                 'Lauf-Geschwindigkeit

'Betriebsartenwahl
'-hier kann die Betriebsart der LCD-Hintergrundbeleuchtung festgelegt werden
'--M_ba = 0: Zhlen mit Durchschalten von 16 Ausgngen (BCD-kodiert)
'--M_ba = 1: gegenlufiges Lauflicht (zusammen)
'--M_ba = 2: gegenlufiges Lauflicht (auseinander)
'--M_ba = 3: Betriebsart 2 und 3 abwechselnd
'--M_ba = 4: Fllen
'--M_ba = 5: Blinken
'--M_ba = 6: Blitzlichter
'--M_ba = 7: Zufall

   If Pind = 255 Then Goto M_001
'-Rcksetzen
   Z_out_int = 0 : Z_out1_byt = 0 : Z_out2_byt = 0

'-Abfrage Port D auf Betriebsartenwechsel
   Select Case Pind
      Case 254                                   'Wert: 255-  1
         M_ba = 0
      Case 253                                   'Wert: 255-  2
         M_ba = 1
      Case 251                                   'Wert: 255-  4
         M_ba = 2
      Case 247                                   'Wert: 255-  8
         M_ba = 3
      Case 239                                   'Wert: 255- 16
         M_ba = 4
      Case 223                                   'Wert: 255- 32
         M_ba = 5
      Case 191                                   'Wert: 255- 64
         M_ba = 6
      Case 127                                   'Wert: 255-128
         M_ba = 7
   End Select
M_001:

   Waitms T_warte                                'Wartezeit
'Auswahl der Betriebsart
   Select Case M_ba
      Case 0
         Incr Z_out_int                          'Weiterzhlen
         Z_out1_byt = Low(z_out_int)             'Ausgnge schalten: LSB
         Z_out2_byt = High(z_out_int)            'Ausgnge schalten: MSB
      Case 1
         Shift Z_out1_byt , Left , 1             'Schieben links  (Faktor 2)
         Shift Z_out2_byt , Right , 1            'Schieben rechts (Faktor 2)
         If Z_out1_byt = 0 Then Z_out1_byt = 1   'Verhalten bei Ende
         If Z_out2_byt = 0 Then Z_out2_byt = 128 'Verhalten bei Ende
      Case 2
         Shift Z_out1_byt , Right , 1            'Schieben links  (Faktor 2)
         Shift Z_out2_byt , Left , 1             'Schieben rechts (Faktor 2)
         If Z_out1_byt = 0 Then Z_out1_byt = 128 'Verhalten bei Ende
         If Z_out2_byt = 0 Then Z_out2_byt = 1   'Verhalten bei Ende
      Case 3
         Incr Z_umschaltung
         If Z_umschaltung < 32 Then
            Shift Z_out1_byt , Left , 1          'Schieben links  (Faktor 2)
            Shift Z_out2_byt , Right , 1         'Schieben rechts (Faktor 2)
            If Z_out1_byt = 0 Then Z_out1_byt = 1       'Verhalten bei Ende
            If Z_out2_byt = 0 Then Z_out2_byt = 128       'Verhalten bei Ende
         Else
            Shift Z_out1_byt , Right , 1         'Schieben links  (Faktor 2)
            Shift Z_out2_byt , Left , 1          'Schieben rechts (Faktor 2)
            If Z_out1_byt = 0 Then Z_out1_byt = 128       'Verhalten bei Ende
            If Z_out2_byt = 0 Then Z_out2_byt = 1       'Verhalten bei Ende
         End If
         If Z_umschaltung = 64 Then
            Z_umschaltung = 0
         End If
      Case 4
         Z_out2_byt = Z_out2_byt / 2
         If Z_out1_byt = 255 Then Z_out1_byt = 0 'Verhalten bei Ende
         If Z_out2_byt = 0 Then Z_out2_byt = 255 'Verhalten bei Ende
         Z_out1_byt = 2 * Z_out1_byt
         If Z_out1_byt = 0 Then
            Z_out1_byt = 1
         Else
            Incr Z_out1_byt
         End If
      Case 5
         Waitms T_warte                          'Wartezeit
         If Z_out1_byt <= 15 Then
            Z_out1_byt = 240
            Z_out2_byt = 15
         Else
            Z_out1_byt = 15
            Z_out2_byt = 240
         End If
      Case 6
         Z_out1_byt = Rnd(17)
         Z_out1_byt = Z_out1_byt / 2
         Z_out1_byt = 2 ^ Z_out1_byt
         Porta = Z_out1_byt                      'Blitzlicht Port 1 Ein
         Z_out2_byt = Rnd(17)
         Z_out2_byt = Z_out2_byt / 2
         Z_out2_byt = 2 ^ Z_out2_byt
         Portc = Z_out2_byt                      'Blitzlicht Port 2 Ein
         Waitms 50
         Z_out1_byt = 0 : Z_out2_byt = 0         'Blitzlicht Port 1/2 Aus
      Case 7                                     'Zufall
         Z_out1_byt = Rnd(256)
         Z_out2_byt = Rnd(100)
         Z_out2_byt = Rnd(256)
   End Select
'-Ausgabe
   Porta = Z_out1_byt                            'Ausgnge schalten
   Portc = Z_out2_byt                            'Ausgnge schalten

Loop
End                                              'Programmende

'-------------------------------------------------------------------------------
'=== U N T E R P R O G R A M M E ===============================================
'-------------------------------------------------------------------------------
'keine

'-------------------------------------------------------------------------------
'=== I N T E R R U P T R O U T I N E N =========================================
'-------------------------------------------------------------------------------

'keine