'Gertesoftware ... : Modulare Kleinsteuerung MKS
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Programm ......... : Beispiele fr den Betrieb einer LCD-Anzeige
' Prog.-Sprache ... : BASCOM
' Prog.-Inhalt .... : - Konfiguration einer LCD-Anzeige (2x 16 Zeichen)
'                     - Helligkeitssteuerung der Hintergrundbeleuchtung
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Programmierer .... : Ingolf Bauer (ingolf.bauer@nexgo.de)
' Prog.-Version ... : 1.0
' Prog.-Datum ..... : 01.03.2005
'-------------------------------------------------------------------------------
'
'Historie
'========
'V 1.0 (nderungen zur Version 0.1):
'- - - - - - - - - - - - - - - - - -
'00. 01.03.2005 - Programmierung
'01. Erforderlicher Programmspeicher des MCs (Programm-Flashspeicher): < 10%
'
'V 0.1
'01. 01.12.2004 - Konzepterarbeitung
'-------------------------------------------------------------------------------
'
'Hinweise
'========
'01.  Allgemeines
'01.1 Im Men von BASCOM (Compiler-Chip) vor dem Compilieren umfangreicher Pro-
'     gramme mindestens einstellen mit:
'     > HW Stack 32   Soft Stack 32   Framesize 32
'     Es kommt sonst zu undefinierten Sprngen oder falschen Anzeigen.
'
'02.  Einstellung der FUSE-Bits:
'02.1 Frequenzeinstellung (extern, Quarz)
'     > CKSEL1 bis CKSEL3 mit Haken versehen
'     Im Lieferzustand hat nur CKSEL0 die Markierung.
'02.2 JTAG(EN)
'     > JTAGEN mit Haken versehen
'     Beim ATmega16 wird das JTAG(EN) Flag defaultmig auf Enable gesetzt. Dies
'     bedeutet, da das JTAG-Interface (prozenahe Datenabfrage) aktiviert ist.
'     Dann kann aber Port C fr keine anderen Aufgaben genutzt werden!
'     Wie und ob das Flag programmig verndert werden kann, ist mir nicht be-
'     kannt. Auf alle Flle kann es ausgeschaltet werden, wenn unter "Configura-
'     tion" beim AVR-Programmer ein Haken bei JTAGEN gesetzt wird.
'
'03.  Anschluschema
'03.1 ATMega16
'     Der Port D ist als digitaler Eingangsport mit 8 Kanlen, die Ports A und
'     C als digitale Ausgangsports mit insgesamt 16 Kanlen initialisiert.
'
'                         --------------|_|--------------
'             LCD-HGB --> | 1 [PB0]             [PA0] 40| >--- DA1.1
'             LCD-HGB --< | 2 [PB1]             [PA1] 39| >--- DA1.2
'             LCD-RS ---< | 3 [PB2]             [PA2] 38| >--- DA1.3
'             LCD-E  ---< | 4 [PB3]             [PA3] 37| >--- DA1.4
'             LCD-DB4 --< | 5 [PB4]             [PA4] 36| >--- DA1.5
'    PRG -- / LCD-DB5 >-< | 6 [PB5, MOSI]       [PA5] 35| >--- DA1.6
'    PRG -- / LCD-DB6 <-< | 7 [PB6, MISO]       [PA6] 34| >--- DA1.7
'    PRG -- / LCD-DB7 >-< | 8 [PB7, SCK]        [PA7] 33| >--- DA1.8
'    PRG -- / Taster ---> | 9 [/Reset]         [AREF] 32| ==== P5V (analog)
'             P5V ======= |10 [VCC]            [AGND] 31| ==== GND (analog)
'             GND ======= |11 [GND]            [AVCC] 30| ==== P5V (analog)
'             Quarz +++++ |12 [XTAL2]           [PC7] 29| >--- DA2.8
'             Quarz +++++ |13 [XTAL1]           [PC6] 28| >--- DA2.7
'            DE1.1 -----> |14 [PD0]             [PC5] 27| >--- DA2.6
'            DE1.2 -----> |15 [PD1]             [PC4] 26| >--- DA2.5
'            DE1.3 -----> |16 [PD2]             [PC3] 25| >--- DA2.4
'            DE1.4 -----> |17 [PD3]             [PC2] 24| >--- DA2.3
'            DE1.5 -----> |18 [PD4]             [PC1] 23| >--- DA2.2
'            DE1.6 -----> |19 [PD5]             [PC0] 22| >--- DA2.1
'            DE1.7 -----> |20 [PD6]             [PD7] 21| <--- DE1.8
'                         -------------------------------
'
'     PRG     ............ : Programmierinterface (parallele Schnittstelle)
'     LCD-xxx ............ : LCD-Anzeige-Modul
'     Quarz .............. : Quarzanschlu (16,0 MHz)
'     P5V/ (analog) ...... : P 5 V der Versorgungsspannung (stabilisiert)
'     GND/ (analog) ...... : Masse der Versorgungsspannung
'
'04.  I/O-Ports
'04.1 Abkrzungen
'     DAx.y .............. : digitaler Ausgang Port.Kanal
'     DEx.y .............. : digitaler Eingang Port.Kanal
'     LCD ................ : LCD-Anzeige
'     PRG ................ : Programmierung ber LPT1 mit Adapterkabel
'     HGB ................ : Hintergrundbeleuchtung der LCD-Anzeige
'
'04.2 Belegung der AVR-Anschlsse
'     >Port
'     Port A  Pin  Typ | Gert  Signal ||
'     --------------------------------------------------------
'      PA0     40  DA  | -      extern ||
'      PA1     39  DA  | -      extern ||
'      PA2     38  DA  | -      extern ||
'      PA3     37  DA  | -      extern ||
'      PA4     36  D4  | -      extern ||
'      PA5     35  DA  | -      extern ||
'      PA6     34  DA  | -      extern ||
'      PA7     33  DA  | -      extern ||
'
'     Port B  Pin  Typ | Gert  Signal ||  Typ | Gert  Signal
'     --------------------------------------------------------
'      PB0     1   DA  | LCD    HGB    ||
'      PB1     2   DE  | LCD    HGB    ||
'      PB2     3   DA  | LCD    RS     ||
'      PB3     4   DA  | LCD    E      ||
'      PB4     5   DA  | LCD    DB4    ||
'      PB5     6   DA  | LCD    DB5    ||  DE  | PRG    MOSI
'      PB6     7   DA  | LCD    DB6    ||  DA  | PRG    MISO
'      PB7     8   DA  | LCD    DB7    ||  DE  | PRG    SCK
'
'     Port C  Pin  Typ | Gert  Signal ||
'     --------------------------------------------------------
'      PC0     22  DA  | -      extern ||
'      PC1     23  DA  | -      extern ||
'      PC2     24  DA  | -      extern ||
'      PC3     25  DA  | -      extern ||
'      PC4     26  D4  | -      extern ||
'      PC5     27  DA  | -      extern ||
'      PC6     28  DA  | -      extern ||
'      PC7     29  DA  | -      extern ||
'
'     Port D  Pin  Typ | Gert  Signal ||
'     --------------------------------------------------------
'      PD0     14  DE  | -      extern ||
'      PD1     15  DE  | -      extern ||
'      PD2     16  DE  | -      extern ||
'      PD3     17  DE  | -      extern ||
'      PD4     18  DE  | -      extern ||
'      PD5     19  DE  | -      extern ||
'      PD6     20  DE  | -      extern ||
'      PD7     21  DE  | -      extern ||
'
'     Sonstige
'             Pin  Typ | Gert  Signal ||  Typ | Gert  Signal
'     --------------------------------------------------------
'              9   DE  | Taster /RESET ||   DE | PRG    /RESET
'-------------------------------------------------------------------------------
'
'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'                          P R O G R A M M S T A R T
'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'
'Definition
   $regfile = "m16def.dat"                       'Prozessor ... : ATMega16
   $crystal = 16000000                           'Quarz ....... : 16,0 MHz
'-------------------------------------------------------------------------------
'
'
'         E I N S T E L L U N G   F  R   D E N   T E S T B E T R I E B
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Achtung:
'========
' Soll der Compiler fr den Simulationsbetrieb Wartezeiten bergehen, kann im
' Testbetrieb der folgende Befehl ($sim) aktiviert werden.
' $sim
'
'
'LCD Konfiguration
'=LCD-Typ ..... : 2 Zeilen, 16 Zeichen
'-Datenport ... : Port B wird neben den Anschlssen zur Programmierung auch
'                 zur Datenbertragung ans LCD-Modul benutzt
   Config Lcd = 16 * 2
   Config Lcdpin = Pin , Db4 = Portb.4 , Db5 = Portb.5 , Db6 = Portb.6 , _
          Db7 = Portb.7 , E = Portb.3 , Rs = Portb.2
   Cls

'Timer fr die Helligkeitssteuerung der LCD-Hintergrundbeleuchtung (HGB)
'=Timer0 (T0), Interrupt 1 (INT1) wird ausgelst beim berlauf (256, da 8 Bit)
   Config Timer0 = Timer , Prescale = 8          'Konfiguration Timer0 (T0)
   On Timer0 It0_hgb                             'Interrupt: T0-berlauf
   Enable Timer0                                 'Interrupt: T0-Einschalten
   Enable Interrupts                             'Interrupt: global zulassen

'Deklaration
'=Variable
'-Flag/ Zhler/ Merker
   Dim M_ba As Byte                              'Merker: Betriebsart

   Dim Z_hgb As Integer                          'Zhler: Hintergrundbeleuchtung

   Dim M_hgb As Integer                          'Merker: Helligkeit
   Dim M_01 As Byte                              'Merker: Hilfsmerker 01

'Initialisierung
'=I/O-Ports
'==Port A
'-digitale Ausgnge (Port 1)
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
   Ddra = &B1111_1111                            'PA0-7 Aus
'-Pullup-Widerstand (aktiv [1], passiv [0])
'--wird bentigt, wenn Initialisierung als Eingangsport
'   Porta = &B0000_0000                           'PA0-7 passiv

'==Port B
'-LCD Anzeige mit Hintergrundbeleuchtung (Z_hgb) und Programmierschnittstelle
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
   Ddrb = &B1111_1111                            'PB0-7 Aus
'-Pullup-Widerstand (aktiv [1], passiv [0])
'--wird bentigt, wenn Initialisierung als Eingangsport
'   Portb = &B0000_0000                           'PB0-7 passiv

'==Port C
'-digitale Ausgnge (Port 2)
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
   Ddrc = &B1111_1111                            'PC0-7 Aus
'-Pullup-Widerstand (aktiv [1], passiv [0])
'--wird bentigt, wenn Initialisierung als Eingangsport
'   Portc = &B0000_0000                           'PC0-7 passiv

'==Port D
'-digitale Eingnge (Port 3)
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
   Ddrd = &B0000_0000                            'PD0-7 Ein
'-Pullup-Widerstand (aktiv [1], passiv [0])
   Portd = &B1111_1111                           'PD0-7 aktiv

'=Werte
   M_ba = 0                                      'Startwert


'Programm
Do
'Betriebsartenwahl
'-hier kann die Betriebsart der LCD-Hintergrundbeleuchtung festgelegt werden
'--M_ba = 0: feste Einstellung auf 20%
'--M_ba = 1: schrittweise Erhhung von 0 auf 100% im Sekundentakt
'--M_ba = 2: Wechsel zwischen 0% und 50% im 3-Sekundentakt
'--M_ba = 3-7: nicht belegt

   If Pind = 255 Then Goto M_001
      Select Case Pind
         Case 254                                'Wert: 255-  1
            M_ba = 0
         Case 253                                'Wert: 255-  2
            M_ba = 1
         Case 251                                'Wert: 255-  4
            M_ba = 2
         Case 247                                'Wert: 255-  8
            M_ba = 3
         Case 239                                'Wert: 255- 16
            M_ba = 4
         Case 223                                'Wert: 255- 32
            M_ba = 5
         Case 191                                'Wert: 255- 64
            M_ba = 6
         Case 127                                'Wert: 255-128
            M_ba = 7
      End Select
      Gosub Up_ba
M_001:

'Art der LCD-Hintergrundbeleuchtung entsprechend eingestellter Betriebsart
   Select Case M_ba
      Case 0
         M_hgb = 20                              '20% Helligkeit
         Gosub Up_anzeige                        'Text fr LCD-Anzeige
      Case 1
         Wait 1                                  'Wartezeit: 1 s
         M_hgb = M_hgb + 10                      'Schrittweise Erhhung
         If M_hgb >= 110 Then M_hgb = 0          'Wert auf 0 stellen
         Gosub Up_anzeige                        'Text fr LCD-Anzeige
      Case 2
         Wait 3                                  'Wartezeit: 3 s
         M_hgb = M_hgb + 50                      '50% Helligkeit
         If M_hgb >= 100 Then M_hgb = 0          'Wert auf 0 stellen
         Gosub Up_anzeige                        'Text fr LCD-Anzeige
'-Bereich nicht belegt
      Case 3                                     'nicht belegt
      Case 4                                     'nicht belegt
      Case 5                                     'nicht belegt
      Case 6                                     'nicht belegt
      Case 7                                     'nicht belegt
      Case 8                                     'nicht belegt
   End Select
Loop
End                                              'Programmende

'-------------------------------------------------------------------------------
'=== U N T E R P R O G R A M M E ===============================================
'-------------------------------------------------------------------------------
Up_anzeige:
'Text fr LCD-Anzeige
'-LCD-Anzeige: Helligkeitswert
   Cursor Off
   Locate 1 , 1 : Lcd "Helligkeit:     "         'LCD-Anzeige: 1. Zeile
   Locate 2 , 1 : Lcd M_hgb ; " %           "    '             2. Zeile
Return

Up_ba:
'neue Betriesart
   Portb.1 = 0                                   'LED der HGB: Aus
   Z_hgb = 0                                     'Zhler rcksetzen
   M_hgb = 0                                     'Merker rcksetzen
   Locate 1 , 1 : Lcd "  Bitte warten "
   Locate 2 , 1 : Lcd "  Umschaltung   "
   Waitms 500
Return
'-------------------------------------------------------------------------------
'=== I N T E R R U P T R O U T I N E N =========================================
'-------------------------------------------------------------------------------
'-01. Einstellen des Wertes fr die LCD-Hintergrundbeleuchtung (HGB)
It0_hgb:
'Interrupt wird durch berlauf (256) von Timer 0 ausgelst
'-Hintergrundbeleuchtung
   Incr Z_hgb                                    'Zhler-Prozentwert HGB erhhen
   Select Case Z_hgb
      Case 100                                   '100% Helligkeit erreicht
          Portb.1 = 0                            'LED der HGB: Aus
          Z_hgb = 0                              'Zhler rcksetzen
      Case Is <= M_hgb                           'x%: Ansteuerung ein
          Portb.1 = 1                            'Led Der Hgb : Ein
      Case Is > M_hgb                            '100-x%: Ansteuerung aus
          Portb.1 = 0
   End Select
Return